@@ -8,9 +8,12 @@ from datetime import datetime, timedelta
 import asyncio
 import re
 import pytz
-import jq
+import jello
+from jello.lib import opts, load_json, pyquery, Schema, Json
+import inspect
 
 from .data_sources import DATA_SOURCES
+from .event_payload import event_payload
 
 _logger = logging.getLogger(__name__)
 
@@ -28,9 +31,10 @@ ENABLED_DATA_SOURCES = list()
 
 
 class StatisticHandler(object):
-    def __init__(self, protocol):
+    def __init__(self, protocol, event_listeners=None):
         super().__init__()
         self.__protocol = protocol
+        self.__event_listeners = event_listeners
         self.__statistics = {}
 
     @property
@@ -61,11 +65,17 @@ class StatisticHandler(object):
                     if len(ENABLED_DATA_SOURCES) > 0:
                         if statistic_id in ENABLED_DATA_SOURCES:
                             self.__statistics[statistic_id] = Statistic(
-                                handler=self, id=statistic_id, data=statistic_reference
+                                handler=self,
+                                id=statistic_id,
+                                data=statistic_reference,
+                                event_listeners=self.__event_listeners,
                             )
                     else:
                         self.__statistics[statistic_id] = Statistic(
-                            handler=self, id=statistic_id, data=statistic_reference
+                            handler=self,
+                            id=statistic_id,
+                            data=statistic_reference,
+                            event_listeners=self.__event_listeners,
                         )
             page += 1
             all_statistics = await self.__listall(limit=limit, page=page)
@@ -132,6 +142,9 @@ class StatisticHandler(object):
             reference = get["soup"].select_one(statistic.selector)
 
             if not reference:
+                _logger.error(
+                    "%s could not find selector %s", statistic.id, statistic.selector
+                )
                 return statistic
 
             value = reference.string
@@ -144,24 +157,41 @@ class StatisticHandler(object):
             reference = get["dom"].xpath(statistic.xpath)
 
             if not reference and reference[0]:
+                _logger.error(
+                    "%s could not find xpath %s", statistic.id, statistic.selector
+                )
                 return statistic
 
             value = reference[0]
 
         if (
-            hasattr(statistic, "jq")
-            and statistic.jq is not None
+            hasattr(statistic, "json_search")
+            and statistic.json_search is not None
             and get["json_data"] is not None
         ):
+            # https://blog.kellybrazil.com/2020/03/25/jello-the-jq-alternative-for-pythonistas/
+            # load the JSON or JSON Lines
+            list_dict_data = None
             try:
-                value = jq.compile(statistic.jq).input(get["json_data"]).first()
-                if statistic.typeName == "integer":
-                    if value is None:
-                        value = 0
-            except Exception as err:
-                _logger.error(err)
+                list_dict_data = load_json(get["body"])
+            except Exception as e:
+                msg = f"""JSON Load Exception: Cannot parse the data (Not valid JSON or JSON Lines)
+            {e}
+            """
+                _logger.error(f"jello:  {msg}")
                 return statistic
 
+            try:
+                value = pyquery(list_dict_data, statistic.json_search)
+                if type(value) == list:
+                    value = value[0]
+            except Exception as e:
+                _logger.exception(e)
+                return statistic
+
+            if statistic.typeName == "integer" and value is None:
+                value = 0
+
         if (
             hasattr(statistic, "regex")
             and statistic.regex is not None
@@ -255,14 +285,12 @@ class StatisticHandler(object):
 
 
 class Statistic(object):
-    def __init__(self, handler=None, id=None, data=None):
+    def __init__(self, handler=None, id=None, data=None, event_listeners=None):
         super().__init__()
 
-        if handler:
-            self.__handler = handler
-
-        if id:
-            self.__id = id
+        self.__handler = handler
+        self.__id = id
+        self.__event_listeners = event_listeners
 
         if data:
             if "host" in data:
@@ -279,8 +307,8 @@ class Statistic(object):
                 self.__selector = data["selector"]
             if "xpath" in data:
                 self.__xpath = data["xpath"]
-            if "jq" in data:
-                self.__jq = data["jq"]
+            if "json_search" in data:
+                self.__json_search = data["json_search"]
             if "regex" in data:
                 self.__regex = data["regex"]
             if "typeId" in data:
@@ -307,26 +335,58 @@ class Statistic(object):
         if not self.__id:
             return
         await self.__handler.details(self.__id)
-        if self.changed and event_receiver is not None:
-            try:
-                event_receiver(
-                    event_type="statistic_updated",
-                    statistic_id=self.id,
-                    statistic=self,
-                    ts=self.updated,
-                )
-                _logger.debug(
-                    "Change sent to event handler for %s (%s)",
-                    self.name,
-                    self.id,
-                )
-            except Exception as err:
-                _logger.error(
-                    "Failed to send event to event receiver %s (%s)",
-                    self.name,
-                    self.id,
-                )
-                _logger.exception(err)
+        if self.changed:
+
+            payload = event_payload("statistic_updated", self, self.updated)
+
+            if type(self.__event_listeners) == list and self.__event_listeners:
+                for event_listener in self.__event_listeners:
+                    if callable(event_listener):
+                        _logger.debug(
+                            "Event triggered for %s to %s",
+                            self.id,
+                            event_listener.__name__,
+                        )
+                        try:
+                            if inspect.iscoroutinefunction(event_listener):
+                                await event_listener(payload)
+                            else:
+                                event_listener(payload)
+                        except Exception as err:
+                            _logger.error(
+                                "Event trigger failed for %s to %s",
+                                self.id,
+                                event_listener.__name__,
+                            )
+                            _logger.exception(err)
+                    else:
+                        _logger.warning(
+                            "An uncallable event listener exists of type %s",
+                            type(event_listener),
+                        )
+
+            if event_receiver is not None:
+                if callable(event_receiver):
+                    _logger.debug(
+                        "Event triggered for %s to %s", self.id, event_receiver.__name__
+                    )
+                    try:
+                        if inspect.iscoroutinefunction(event_receiver):
+                            await event_receiver(payload)
+                        else:
+                            event_receiver(payload)
+                    except Exception as err:
+                        _logger.error(
+                            "Event trigger failed for %s to %s",
+                            self.id,
+                            event_receiver.__name__,
+                        )
+                        _logger.exception(err)
+                else:
+                    _logger.warning(
+                        "An uncallable event receiver exists of type %s",
+                        type(event_receiver),
+                    )
 
     @property
     def id(self):
@@ -465,9 +525,9 @@ class Statistic(object):
             return None
 
     @property
-    def jq(self):
+    def json_search(self):
         try:
-            return self.__jq
+            return self.__json_search
         except AttributeError:
             return None
 
